@@ -3,7 +3,7 @@
 #
 # COPYRIGHT:
 #
-# This software is Copyright (c) 1996-2016 Best Practical Solutions, LLC
+# This software is Copyright (c) 1996-2017 Best Practical Solutions, LLC
 #                                          <sales@bestpractical.com>
 #
 # (Except where explicitly superseded by other copyright notices)
@@ -929,11 +929,6 @@ sub _AddMember {
         return(0, $self->loc("Group not found"));
     }
 
-    unless ($new_member =~ /^\d+$/) {
-        $RT::Logger->crit("_AddMember called with a parameter that's not an integer.");
-    }
-
-
     my $new_member_obj = RT::Principal->new( $self->CurrentUser );
     $new_member_obj->Load($new_member);
 
@@ -996,6 +991,19 @@ sub _AddMember {
             unless $ok;
     }
 
+    # Record transactions for UserDefined groups
+    if ($args{RecordTransaction} && $self->Domain eq 'UserDefined') {
+        $new_member_obj->Object->_NewTransaction(
+            Type  => 'AddMembership',
+            Field => $self->PrincipalObj->id,
+        );
+
+        $self->_NewTransaction(
+            Type  => 'AddMember',
+            Field => $new_member,
+        );
+    }
+
     # Record an Add/SetWatcher txn on the object if we're a role group
     if ($args{RecordTransaction} and $self->RoleClass) {
         my $obj = $args{Object} || $self->RoleGroupObject;
@@ -1138,11 +1146,7 @@ sub DeleteMember {
     $self->_DeleteMember($member_id, @_);
 }
 
-# A helper subroutine for DeleteMember that bypasses the ACL checks
-# this should _ONLY_ ever be called from Ticket/Queue  DeleteWatcher
-# when we want to deal with groups according to queue rights
-# In the dim future, this will all get factored out and life
-# will get better
+# A helper subroutine for DeleteMember that bypasses the ACL checks.
 
 sub _DeleteMember {
     my $self = shift;
@@ -1152,42 +1156,35 @@ sub _DeleteMember {
         @_,
     );
 
-
     my $member_obj =  RT::GroupMember->new( $self->CurrentUser );
-    
-    $member_obj->LoadByCols( MemberId  => $member_id,
-                             GroupId => $self->PrincipalId);
+    $member_obj->LoadByCols(
+        MemberId => $member_id,
+        GroupId  => $self->PrincipalId,
+    );
 
-
-    #If we couldn't load it, return undef.
+    # If we couldn't load it, return undef.
     unless ( $member_obj->Id() ) {
         $RT::Logger->debug("Group has no member with that id");
-        return ( 0,$self->loc( "Group has no such member" ));
+        return ( 0, $self->loc( "Group has no such member" ));
     }
 
-    my $old_member = $member_obj->MemberId;
-
-    #Now that we've checked ACLs and sanity, delete the groupmember
-    my $val = $member_obj->Delete();
-
-    unless ($val) {
-        $RT::Logger->debug("Failed to delete group ".$self->Id." member ". $member_id);
-        return ( 0, $self->loc("Member not deleted" ));
-    }
+    # Now that we've checked ACLs and sanity, delete the groupmember
+    my ($ok, $msg) = $member_obj->Delete();
+    return ( 0, $self->loc("Member not deleted" )) unless $ok;
 
     if ($self->RoleClass) {
         my %txn = (
-            OldValue => $old_member,
+            OldValue => $member_id,
             Field    => $self->Name,
         );
 
         if ($self->SingleMemberRoleGroup) {
-            # _AddMember creates the Set-Owner txn (for example) but we handle
-            # the SetWatcher-Owner txn below.
+            # _AddMember creates the Set-Owner txn (for example) but
+            # we handle the SetWatcher-Owner txn below.
             $self->_AddMember(
-                PrincipalId             => RT->Nobody->Id,
-                RecordTransaction       => 0,
-                RecordSetTransaction    => $args{RecordTransaction},
+                PrincipalId          => RT->Nobody->Id,
+                RecordTransaction    => 0,
+                RecordSetTransaction => $args{RecordTransaction},
             );
             $txn{Type}     = "SetWatcher";
             $txn{NewValue} = RT->Nobody->id;
@@ -1201,7 +1198,20 @@ sub _DeleteMember {
         }
     }
 
-    return ( $val, $self->loc("Member deleted") );
+    # Record transactions for UserDefined groups
+    if ($args{RecordTransaction} && $self->Domain eq 'UserDefined') {
+        $member_obj->MemberObj->Object->_NewTransaction(
+            Type  => 'DeleteMembership',
+            Field => $self->PrincipalObj->id,
+        );
+
+        $self->_NewTransaction(
+            Type  => 'DeleteMember',
+            Field => $member_id,
+        );
+    }
+
+    return ( $ok, $self->loc("Member deleted") );
 }
 
 
@@ -1581,6 +1591,12 @@ sub __DependsOn {
     );
     push( @$list, $objs );
 
+# Cleanup group's membership transactions
+    $objs = RT::Transactions->new( $self->CurrentUser );
+    $objs->Limit( FIELD => 'Type', OPERATOR => 'IN', VALUE => ['AddMember', 'DeleteMember'] );
+    $objs->Limit( FIELD => 'Field', VALUE => $self->PrincipalObj->id, ENTRYAGGREGATOR => 'AND' );
+    push( @$list, $objs );
+
     $deps->_PushDependencies(
         BaseObject => $self,
         Flags => RT::Shredder::Constants::DEPENDS_ON,
@@ -1636,13 +1652,12 @@ sub PreInflate {
         return;
     };
 
-    # Go looking for the pre-existing version of the it
+    # Go looking for the pre-existing version of it
     if ($data->{Domain} eq "ACLEquivalence") {
         $obj->LoadACLEquivalenceGroup( $data->{Instance} );
         return $duplicated->() if $obj->Id;
 
-        # Update the name and description for the new ID
-        $data->{Name} = 'User '. $data->{Instance};
+        # Update description for the new ID
         $data->{Description} = 'ACL equiv. for user '.$data->{Instance};
     } elsif ($data->{Domain} eq "UserDefined") {
         $data->{Name} = $importer->Qualify($data->{Name});
