@@ -2,7 +2,7 @@
 #
 # COPYRIGHT:
 #
-# This software is Copyright (c) 1996-2016 Best Practical Solutions, LLC
+# This software is Copyright (c) 1996-2017 Best Practical Solutions, LLC
 #                                          <sales@bestpractical.com>
 #
 # (Except where explicitly superseded by other copyright notices)
@@ -70,6 +70,7 @@ In C<RT_SiteConfig.pm>:
     Set($LDAPHost,'my.ldap.host');
     Set($LDAPUser,'me');
     Set($LDAPPassword,'mypass');
+    Set($LDAPBase, 'ou=People,o=Our Place');
     Set($LDAPFilter, '(&(cn = users))');
     Set($LDAPMapping, {Name         => 'uid', # required
                        EmailAddress => 'mail',
@@ -88,12 +89,10 @@ In C<RT_SiteConfig.pm>:
 Running the import:
 
     # Run a test import
-    /opt/rt4/sbin/rt-ldapimport \
-    --debug > ldapimport.debug 2>&1
+    /opt/rt4/sbin/rt-ldapimport --verbose > ldapimport.debug 2>&1
     
     # Run for real, possibly put in cron
-    /opt/rt4/sbin/rt-ldapimport \
-    --import
+    /opt/rt4/sbin/rt-ldapimport --import
 
 =head1 CONFIGURATION
 
@@ -347,7 +346,7 @@ members) should exist unless there are inconsistencies in your LDAP configuratio
 
 Executing C<rt-ldapimport> will run a test that connects to your LDAP server
 and prints out a list of the users found. To see more about these users,
-and to see more general debug information, include the C<--debug> flag.
+and to see more general debug information, include the C<--verbose> flag.
 
 That debug information is also sent to the RT log with the debug level.
 Errors are logged to the screen and to the RT log.
