@@ -2,7 +2,7 @@
 #
 # COPYRIGHT:
 #
-# This software is Copyright (c) 1996-2016 Best Practical Solutions, LLC
+# This software is Copyright (c) 1996-2017 Best Practical Solutions, LLC
 #                                          <sales@bestpractical.com>
 #
 # (Except where explicitly superseded by other copyright notices)
@@ -73,6 +73,8 @@ sub parse_options {
     $self->{server} ||= $self->loader->guess;
 
     my %args = @{$self->{options}};
+    $self->{webpath} ||= $args{webpath} if $args{webpath};
+
     if ($self->{server} eq "FCGI") {
         # We deal with the possible failure modes of this in ->run
     } elsif ($args{port}) {
@@ -138,6 +140,13 @@ sub run {
         exit 1;
     }
 
+    if ( $self->{webpath} ){
+        require Plack::App::URLMap;
+        my $urlmap = Plack::App::URLMap->new;
+        $urlmap->map($self->{webpath} => $self->{app});
+        $self->{app} = $urlmap->to_app;
+    }
+
     eval { $self->SUPER::run(@_) };
     my $err = $@;
     exit 0 unless $err;
