@@ -2,7 +2,7 @@
 #
 # COPYRIGHT:
 #
-# This software is Copyright (c) 1996-2016 Best Practical Solutions, LLC
+# This software is Copyright (c) 1996-2017 Best Practical Solutions, LLC
 #                                          <sales@bestpractical.com>
 #
 # (Except where explicitly superseded by other copyright notices)
@@ -112,9 +112,28 @@ No API available for deleting things just yet.
 
 =cut
 
-sub Delete  {
+sub Delete {
     my $self = shift;
-    return(0, $self->loc('Unimplemented'));
+
+    unless ( $self->CurrentUser->HasRight( Object => RT->System, Right => 'ModifyScrips' ) ) {
+        return ( 0, $self->loc('Permission Denied') );
+    }
+
+    my $scrips = RT::Scrips->new( RT->SystemUser );
+    $scrips->Limit( FIELD => 'ScripCondition', VALUE => $self->id );
+    if ( $scrips->Count ) {
+        return ( 0, $self->loc('Condition is in use') );
+    }
+
+    return $self->SUPER::Delete(@_);
+}
+
+sub UsedBy {
+    my $self = shift;
+
+    my $scrips = RT::Scrips->new( $self->CurrentUser );
+    $scrips->Limit( FIELD => 'ScripCondition', VALUE => $self->Id );
+    return $scrips;
 }
 
 
