@@ -2,7 +2,7 @@
 #
 # COPYRIGHT:
 #
-# This software is Copyright (c) 1996-2016 Best Practical Solutions, LLC
+# This software is Copyright (c) 1996-2017 Best Practical Solutions, LLC
 #                                          <sales@bestpractical.com>
 #
 # (Except where explicitly superseded by other copyright notices)
@@ -81,7 +81,7 @@ sub Squish {
         my $res = RT::Interface::Web::Handler->GetStatic($uri);
 
         if ($res->is_success) {
-            $content .= $res->decoded_content;
+            $content .= $res->decoded_content . "\n";
         } else {
             RT->Logger->error("Unable to fetch $uri for JS Squishing: " . $res->status_line);
             next;
