@@ -2,7 +2,7 @@
 #
 # COPYRIGHT:
 #
-# This software is Copyright (c) 1996-2016 Best Practical Solutions, LLC
+# This software is Copyright (c) 1996-2017 Best Practical Solutions, LLC
 #                                          <sales@bestpractical.com>
 #
 # (Except where explicitly superseded by other copyright notices)
@@ -106,7 +106,7 @@ __PACKAGE__->RegisterCustomFieldJoin(@$_) for
 our %FIELD_METADATA = (
     Status          => [ 'STRING', ], #loc_left_pair
     SLA             => [ 'STRING', ], #loc_left_pair
-    Queue           => [ 'ENUM' => 'Queue', ], #loc_left_pair
+    Queue           => [ 'QUEUE' ], #loc_left_pair
     Type            => [ 'ENUM', ], #loc_left_pair
     Creator         => [ 'ENUM' => 'User', ], #loc_left_pair
     LastUpdatedBy   => [ 'ENUM' => 'User', ], #loc_left_pair
@@ -187,6 +187,7 @@ our %dispatch = (
     LINK            => \&_LinkLimit,
     DATE            => \&_DateLimit,
     STRING          => \&_StringLimit,
+    QUEUE           => \&_QueueLimit,
     TRANSFIELD      => \&_TransLimit,
     TRANSCONTENT    => \&_TransContentLimit,
     TRANSDATE       => \&_TransDateLimit,
@@ -221,6 +222,12 @@ my %DefaultEA = (
         'LIKE'     => 'AND',
         'NOT LIKE' => 'AND'
     },
+    QUEUE => {
+         '='        => 'OR',
+         '!='       => 'AND',
+         'LIKE'     => 'OR',
+         'NOT LIKE' => 'AND'
+    },
     TRANSFIELD   => 'AND',
     TRANSDATE    => 'AND',
     LINK         => 'OR',
@@ -709,6 +716,48 @@ sub _StringLimit {
     );
 }
 
+=head2 _QueueLimit
+
+Handle Queue field supporting both "is" and "match".
+
+Input should be a queue name or a partial string.
+
+=cut
+
+sub _QueueLimit {
+    my ($sb, $field, $op, $value, @rest ) = @_;
+
+    if ($op eq 'LIKE' || $op eq 'NOT LIKE') {
+        my $alias = $sb->{_sql_aliases}{queues} ||= $sb->Join(
+            ALIAS1 => 'main',
+            FIELD1 => 'Queue',
+            TABLE2 => 'Queues',
+            FIELD2 => 'id',
+        );
+
+        return $sb->Limit(
+           ALIAS         => $alias,
+           FIELD         => 'Name',
+           OPERATOR      => $op,
+           VALUE         => $value,
+           CASESENSITIVE => 0,
+           @rest,
+       );
+
+    }
+
+    my $o = RT::Queue->new( $sb->CurrentUser );
+    $o->Load($value);
+    $value = $o->Id || 0;
+    $sb->Limit(
+        FIELD         => $field,
+        OPERATOR      => $op,
+        VALUE         => $value,
+        CASESENSITIVE => 0,
+        @rest,
+    );
+}
+
 =head2 _TransDateLimit
 
 Handle fields limiting based on Transaction Date.
@@ -1339,7 +1388,7 @@ sub OrderByCols {
                 next;
             }
 
-            if ( $meta->[0] eq 'ENUM' && ($meta->[1]||'') eq 'Queue' ) {
+            if ( $meta->[0] eq 'QUEUE' ) {
                 my $alias = $self->Join(
                     TYPE   => 'LEFT',
                     ALIAS1 => 'main',
@@ -3117,6 +3166,9 @@ sub _parser {
             # replace __CurrentUser__ with id
             $value = $self->CurrentUser->id if $value eq '__CurrentUser__';
 
+            # replace __CurrentUserName__ with the username
+            $value = $self->CurrentUser->Name if $value eq '__CurrentUserName__';
+
             my $sub = $dispatch{ $class }
                 or die "No dispatch method for class '$class'";
 
